#!/usr/bin/perl
#
#   Empire - A multi-player, client/server Internet based war game.
#   Copyright (C) 1986-2015, Dave Pare, Jeff Bailey, Thomas Ruschak,
#                 Ken Stevens, Steve McClure, Markus Armbruster
#
#   Empire is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
#   ---
#
#   See files README, COPYING and CREDITS in the root of the source
#   tree for related information and legal notices.  It is expected
#   that future projects/authors will amend these files as needed.
#
#   ---
#
#   mktop.pl: Create the index of subjects
#
#   Known contributors to this file:
#      Ken Stevens (when it was still info.pl)
#      Markus Armbruster, 2006-2013
#

# Usage: mktop.pl OUTFILE SUBJECT...

my $out = shift @ARGV;
my @subject = sort @ARGV;

open(TOP, ">$out")
    or die "Can't open $out: $!";
print TOP <<EOF;
.\\\" DO NOT EDIT THIS FILE.  It was automatically generated by mktop.pl
.TH Info "List of Subjects"
.s1
Empire info is available on the following subjects:
.NF
EOF

# reorder subjects for display in three columns
my $k = 0;
for my $i (0..2) {
    for (my $j = $i; $j <= $#subject; $j += 3) {
	$colsubj[$j] = $subject[$k++];
    }
}

for my $subj (@colsubj) {
    push(@subj, $subj);
    if ($#subj > 1) {
	flush_subj(@subj);
	@subj = ();
    }
}
flush_subj(@subj);
print TOP <<EOF;
.FI
Type "info <Subject>" where <Subject> is one of the subjects listed above.
For a complete list of all info topics, type "info all".
EOF
close TOP;

# Print a row of subjects to TOP
sub flush_subj {
    return unless $#_ >= 0;
    print TOP "  ";
    for (@_) {
	printf TOP "%-25s", $_;
    }
    print TOP "\n";
}
